# Designed to work with GNU make
# Targets:
#
#	make unittest
#		Build mango.io package, and create src/obj/mango
# NOTES:
#
#	2004-06-09 -- update makefile to work properly on Linux
#       2004-11-26 -- added libdl.so to the compile line (-ldl) for
#		      linux dynamic loader support - John Reimer

DFLAGS=-version=Posix

#DMD =		/dmd/bin/dmd
DMD =		dmd
OBJ =		obj/

MANGO =         mango
EXAMPLE =       example/
BASE =          $(MANGO)/base/
TEST =          $(MANGO)/test/
UTILS =         $(MANGO)/utils/
UTILSMODEL =    $(MANGO)/utils/model/
IO =            $(MANGO)/io/
IOMODEL =       $(MANGO)/io/model/
CACHE =         $(MANGO)/cache/
CACHEMODEL =    $(MANGO)/cache/model/
SERVERMODEL =   $(MANGO)/http/server/model/
HTTPCLIENT =    $(MANGO)/http/client/
HTTP =          $(MANGO)/http/
HTTPUTILS =     $(MANGO)/http/utils/
HTTPSERVER =    $(MANGO)/http/server/
SERVLET =       $(MANGO)/servlet/
SERVLETMODEL =  $(MANGO)/servlet/model/
LOG =           $(MANGO)/log/
LOGMODEL =      $(MANGO)/log/model/
CLUSTER =       $(MANGO)/cluster/
CLUSTERMODEL =  $(MANGO)/cluster/model/
CLUSTERSOCKET = $(MANGO)/cluster/qos/socket/
ICU =           $(MANGO)/icu/
FORMAT =        $(MANGO)/format/

COMPILE = 	$(DMD) -c $(DFLAGS) -od$(OBJ) $**

.obj.d :
	$(COMPILE)


SRC_EXAMPLE = \
        $(EXAMPLE)alert.d\
        $(EXAMPLE)cclient.d\
        $(EXAMPLE)cserver.d\
        $(EXAMPLE)filecat.d\
        $(EXAMPLE)filecopy.d\
        $(EXAMPLE)lineio.d\
        $(EXAMPLE)filescan.d\
        $(EXAMPLE)logging.d\
        $(EXAMPLE)pickle.d\
        $(EXAMPLE)randomio.d\
        $(EXAMPLE)servlets.d\
        $(EXAMPLE)tserver.d\

SRC_FORMAT = \
        $(FORMAT)Int.d \
        $(FORMAT)Long.d \
        $(FORMAT)Double.d \
        $(FORMAT)Number.d \
        $(FORMAT)DateTime.d \
        $(FORMAT)DGDouble.d \
        $(FORMAT)Formatter.d \

SRC_IO = \
        $(BASE)System.d \
        $(BASE)ByteSwap.d \
        $(UTILS)Timer.d \
        $(UTILS)Text.d \
        $(UTILS)Random.d \
        $(UTILS)HeapSlice.d \
        $(UTILS)AbstractServer.d \
        $(UTILS)ServerThread.d \
        $(UTILSMODEL)IServer.d \
        $(IOMODEL)IBuffer.d \
        $(IOMODEL)IResource.d \
        $(IOMODEL)IConduit.d \
        $(IOMODEL)IReader.d \
        $(IOMODEL)IWriter.d \
        $(IOMODEL)IPickle.d \
        $(IOMODEL)IBitBucket.d \
        $(IO)ArrayAllocator.d \
        $(IO)TextFormat.d \
        $(IO)TextLayout.d \
        $(IO)Exception.d \
        $(IO)File.d \
        $(IO)FileConst.d \
        $(IO)FileStyle.d \
        $(IO)FilePath.d \
        $(IO)FileScan.d \
        $(IO)ConduitStyle.d \
        $(IO)Conduit.d \
        $(IO)Buffer.d \
        $(IO)Resource.d \
        $(IO)FileSystem.d \
        $(IO)FileConduit.d \
        $(IO)FileProxy.d \
        $(IO)EndianReader.d \
        $(IO)EndianWriter.d \
        $(IO)Token.d \
        $(IO)Tokenizer.d \
        $(IO)TextWriter.d \
        $(IO)TextReader.d \
        $(IO)DisplayWriter.d \
        $(IO)ColumnWriter.d \
        $(IO)Stdin.d \
        $(IO)Stdout.d \
        $(IO)AbstractReader.d \
        $(IO)AbstractWriter.d \
        $(IO)Reader.d \
        $(IO)Writer.d \
        $(IO)Socket.d \
        $(IO)PickleRegistry.d \
        $(IO)PickleReader.d \
        $(IO)PickleWriter.d \
        $(IO)FileBucket.d \
        $(IO)Uri.d \
        $(IO)SocketConduit.d \
        $(IO)ServerSocket.d \
        $(IO)DatagramSocket.d \
        $(IO)MulticastSocket.d \
        $(IO)SocketListener.d \
        $(IO)Properties.d \
        $(IO)Utf8.d \


SRC_CACHE = \
        $(CACHEMODEL)ICache.d \
        $(CACHEMODEL)IPayload.d \
        $(CACHE)HashMap.d \
        $(CACHE)PlainCache.d \
        $(CACHE)Payload.d \
        $(CACHE)QueuedCache.d \
        $(CACHE)VirtualCache.d \


SRC_HTTP = \
        $(SERVERMODEL)IProvider.d \
        $(SERVERMODEL)IProviderBridge.d \
        $(HTTP)HttpReader.d \
        $(HTTP)HttpWriter.d \
        $(HTTPUTILS)TokenStack.d \
        $(HTTPUTILS)Dictionary.d \
        $(HTTPSERVER)HttpTokens.d \
        $(HTTPSERVER)HttpParams.d \
        $(HTTPSERVER)HttpHeaders.d \
        $(HTTPSERVER)HttpCookies.d \
        $(HTTPSERVER)HttpBridge.d \
        $(HTTPSERVER)HttpMessage.d \
        $(HTTPSERVER)HttpRequest.d \
        $(HTTPSERVER)HttpResponse.d \
        $(HTTPSERVER)HttpProvider.d \
        $(HTTPSERVER)HttpServer.d \
        $(HTTPSERVER)HttpThread.d \
        $(HTTPCLIENT)HttpClient.d \


SRC_SERVLET = \
        $(SERVLETMODEL)IServletRequest.d \
        $(SERVLETMODEL)IServletResponse.d \
        $(SERVLET)ServletConfig.d \
        $(SERVLET)Servlet.d \
        $(SERVLET)ServletRequest.d \
        $(SERVLET)ServletResponse.d \
        $(SERVLET)ServletProvider.d \
        $(SERVLET)ServletContext.d \

SRC_LOG = \
        $(LOGMODEL)ILevel.d \
        $(LOGMODEL)ILogger.d \
        $(LOG)Admin.d \
        $(LOG)Hierarchy.d \
        $(LOG)Manager.d \
        $(LOG)Logger.d \
        $(LOG)Event.d \
        $(LOG)Layout.d \
        $(LOG)XmlLayout.d \
        $(LOG)DateLayout.d \
        $(LOG)Appender.d \
        $(LOG)SocketAppender.d \
        $(LOG)Configurator.d \
        $(LOG)FileAppender.d \
        $(LOG)RollingFileAppender.d \
        $(LOG)ConsoleAppender.d \


SRC_CLUSTER = \
        $(CLUSTERMODEL)ICluster.d \
        $(CLUSTERMODEL)IChannel.d \
        $(CLUSTERMODEL)IEvent.d \
        $(CLUSTERMODEL)IMessage.d \
        $(CLUSTERMODEL)IConsumer.d \
        $(CLUSTER)Client.d \
        $(CLUSTER)CacheInvalidator.d \
        $(CLUSTER)CacheInvalidatee.d \
        $(CLUSTER)NetworkCache.d \
        $(CLUSTER)NetworkQueue.d \
        $(CLUSTER)NetworkAlert.d \
        $(CLUSTER)Message.d \
        $(CLUSTER)TaskServer.d \
        $(CLUSTERSOCKET)ProtocolWriter.d \
        $(CLUSTERSOCKET)ProtocolReader.d \
        $(CLUSTERSOCKET)Cluster.d \
        $(CLUSTERSOCKET)RollCall.d \
        $(CLUSTERSOCKET)ClusterEvent.d \
        $(CLUSTERSOCKET)ClusterThread.d \
        $(CLUSTERSOCKET)ClusterServer.d \
        $(CLUSTERSOCKET)ClusterCache.d \


SRC_ICU = \
        $(ICU)ICU.d \
        $(ICU)UChar.d \
        $(ICU)ULocale.d \
        $(ICU)UConverter.d \
        $(ICU)UString.d \
        $(ICU)USearch.d \
        $(ICU)UTimeZone.d \
        $(ICU)UCalendar.d \
        $(ICU)UDateFormat.d \
        $(ICU)UNumberFormat.d \
        $(ICU)UBreakIterator.d \
        $(ICU)UResourceBundle.d \
        $(ICU)UMessageFormat.d \
        $(ICU)UTransform.d \
        $(ICU)UNormalize.d \
        $(ICU)UStringPrep.d \
        $(ICU)UDomainName.d \
        $(ICU)UMango.d \
        $(ICU)USet.d \
        $(ICU)UCollator.d \
        $(ICU)UEnumeration.d \
        $(ICU)URegex.d \

		
libmango.a :
	$(DMD) -c $(DFLAGS) -od$(OBJ) $(SRC_IO) $(SRC_CACHE) $(SRC_HTTP) $(SRC_LOG) $(SRC_CLUSTER) $(SRC_ICU) $(SRC_SERVLET) $(SRC_FORMAT)
	ar -r $@ $(OBJ)*.o

unittest : libmango.a
	$(DMD) -c $(DFLAGS) $(TEST)unittest.d 
	$(CC) -o mangotest unittest.o libmango.a -lphobos -lpthread -ldl -lm -g
	
servlets : libmango.a
	$(DMD) -c $(DFLAGS) $(EXAMPLE)servlets.d 
	$(CC) -o servlets servlets.o libmango.a -lphobos -lpthread -lm -g
	
cclient : libmango.a
	$(DMD) -c $(DFLAGS) $(EXAMPLE)cclient.d 
	$(CC) -o cclient cclient.o libmango.a -lphobos -lpthread -lm -g
	
cserver : libmango.a
	$(DMD) -c $(DFLAGS) $(EXAMPLE)cserver.d 
	$(CC) -o cserver cserver.o libmango.a -lphobos -lpthread -lm -g
	
unittestbin :
	$(DMD) -of./obj/mango $(OBJ)*.o
	
