/*******************************************************************************

        @file DateTime.d
        
        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        
        @version        Initial version, April 2004      
        @author         Kris


*******************************************************************************/

module mango.format.DateTime;

private import  std.date;

private import  mango.format.Int,
                mango.format.Formatter;

/******************************************************************************

******************************************************************************/

class DateTime : Formatter
{
        /**********************************************************************

        **********************************************************************/

        final static tChar[] format (long i)
        {
                return format (new tChar[40], i);
        }

        /**********************************************************************

                Shamelessly plundered from std.date (with permission), this 
                version avoids allocating memory.

                Returns a populated slice of the provided buffer; with zero
                length if the date was invalid.

                Note that this is just a stopgap implementation for now.

        **********************************************************************/

        final static tChar[] format (tChar[] buffer, long time)
        {
                // "Tue Apr 02 02:04:57 GMT-0800 1996"
                assert(buffer.length >= 29 + 7 + 1);

                d_time  t,
                        dst,
                        offset;
                tChar   sign;
                tChar*  p = buffer;

                void str (tChar* s, tChar c)
                {
                        p[0..3] = s[0..3];
                        p += 3;
                        *p++ = c;
                }

                void num (int i, tChar c=0)
                {
                        p += Int.format (p[0..2], i, Int.Radix.Decimal, '0').length;
                        if (c)
                            *p++ = c;
                }
                

                if (time == d_time_nan)
                    return buffer[0..0];

                dst = DaylightSavingTA (time);
                offset = LocalTZA + dst;
                t = time + offset;

                str (&daystr[WeekDay(t) * 3], ' ');
                str (&monstr[MonthFromTime(t) * 3], ' ');
                num (DateFromTime(t), ' ');
                num (HourFromTime(t), ':');
                num (MinFromTime(t), ':');
                num (SecFromTime(t), ' ');

                sign = '+';
                if (offset < 0)
                   {
                   sign = '-';
                   offset = -(LocalTZA + dst);
                   }
                str ("GMT", sign);

                int mn = cast(int)(offset / msPerMinute);
                num (mn / 60);
                num (mn % 60, ' ');

                Int.format (p[0..4], YearFromTime(t));
                p += 4;
        
                return buffer[0..p-buffer.ptr];
        }

        /**********************************************************************

                Convert a string to a date. This is just a symmetrical 
                wrapper around the Phobos functionality.

                Note that this is just a stopgap implementation for now.
              
        **********************************************************************/

        final static long parse (tChar[] date, uint* ate=null)
        in {
           assert (ate == null);  // for now ...
           }
        body
        {
                return std.date.parse (date);
        }
}





/+
                len = sprintf (buffer, "%.3s %.3s %02d %02d:%02d:%02d GMT%c%02d%02d %d",
                               &daystr[WeekDay(t) * 3],
                               &monstr[MonthFromTime(t) * 3],
                               DateFromTime(t),
                               cast(int)HourFromTime(t), cast(int)MinFromTime(t), 
                               cast(int)SecFromTime(t), sign, hr, mn,
                               cast(long)YearFromTime(t));
+/
