/*******************************************************************************

        @file Long.d
        
        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        
        @version        Initial version, Feb 2005
        @author         Kris


*******************************************************************************/

module mango.format.Long;

private import  mango.format.Int,
                mango.format.Number;

/******************************************************************************

        A set of functions for converting between string and numeric 
        values. 

        Used by modules Token and DisplayWriter

******************************************************************************/

class Long : Number
{
        /**************************************************************

        **************************************************************/

        final static tChar[] format (long i)
        {
                return format (new tChar[32], i);
        }


        /**************************************************************

        **************************************************************/

        final static tChar[] format (ulong i, Radix radix, tChar fill = tChar.init)
        {
                return format (new tChar[64], i, radix, fill);
        }


        /**********************************************************************

        **********************************************************************/

        final static tChar[] format (tChar[] dst, long i)
        in {
           assert (dst.length);
           }
        body
        {
                if (i >= 0)
                    return format (dst, i, Radix.Decimal);

                uint len = dst.length - format (dst[1..length], -i, Radix.Decimal).length;
                dst [--len] = '-';
                return dst [len..length];
        }


        /**********************************************************************

        **********************************************************************/

        final static tChar[] format (tChar[] dst, ulong i, Radix radix, tChar fill = tChar.init)
        {
                if (i <= uint.max)
                    return Int.format (dst, i, radix, fill);

                uint len = dst.length;
                char* p = dst.ptr + len;

                if (len)
                    switch (radix)
                           {
                           case Radix.Binary:
                                do {
                                   *--p = '0' + (i & 1);
                                   } while ((i >>>= 1) && --len);
                                break;

                           case Radix.Octal:
                                do {
                                   *--p = '0' + (i & 7);
                                   } while ((i >>>= 3) && --len);
                                break;

                           case Radix.Hexadecimal:
                                 do {
                                    char c = i & 0x0f;
                                    *--p = c + ((c < 10) ? '0' : ('a' - 10));
                                    } while ((i >>= 4) && --len);
                                break;

                           default:
                                do {
                                   *--p = '0' + (i % 10);
                                   } while ((i /= 10) && --len);
                                break;
                           }

                if (! len)
                      throw error;

                if (fill != tChar.init)
                    while (p > dst.ptr)
                           *--p = fill;

                return dst[p-dst.ptr..length];                              
        }


        /**********************************************************************

        **********************************************************************/

        final static ulong parse (tChar[] src, Radix radix=Radix.Decimal, uint* ate=null)
        {
                bool  sign;
                uint  count = trim (src, sign, &radix);

                ulong value;
                foreach (tChar c; src[count..length])
                        {
                        if (c >= '0' && c <= '9')
                           {}
                        else
                           if (c >= 'a' && c <= 'f')
                               c -= 39;
                           else
                              if (c >= 'A' && c <= 'F')
                                  c -= 7;
                              else
                                 break;

                        value = value * radix + (c - '0');
                        ++count;
                        }

                if (ate)
                    *ate = count;
                return sign ? -value : value;
        }
}


