/*******************************************************************************

        @file EndianWriter.d
        
        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, March 2004      
        @author         Kris


*******************************************************************************/

module mango.io.EndianWriter;

public  import  mango.io.Writer;

private import  mango.base.ByteSwap;

/*******************************************************************************

*******************************************************************************/

class EndianWriter : Writer
{       
        typedef void function (void* dst, uint count) Mutator;

        /***********************************************************************
        
                Construct EndianWriter upon the given IBuffer

        ***********************************************************************/

        this (IBuffer buffer)
        {
                super (buffer);

                encode.int16   = 
                encode.int16u  = 
                encode.char16   = &bits16;

                encode.int32   = 
                encode.int32u  = 
                encode.float32 = 
                encode.char32   = &bits32;

                encode.int64   = 
                encode.int64u  = 
                encode.float64 = &bits64;

                encode.float80 = &bits80;
        }

        /***********************************************************************
        
        ***********************************************************************/

        void bits16 (void* dst, uint count)
        {               
                write (dst, count, ~1, &ByteSwap.swap16);   
        }

        /***********************************************************************
        
        ***********************************************************************/

        void bits32 (void* dst, uint count)
        {      
                write (dst, count, ~3, &ByteSwap.swap32);   
        }

        /***********************************************************************
        
        ***********************************************************************/

        void bits64 (void* dst, uint count)
        {      
                write (dst, count, ~7, &ByteSwap.swap64);   
        }

        /***********************************************************************
        
        ***********************************************************************/

        void bits80 (void* dst, uint count)
        {      
                write (dst, count, ~15, &ByteSwap.swap80);   
        }

        /***********************************************************************
        
        ***********************************************************************/

        private void write (void* src, uint count, int mask, Mutator mutate)
        {
                int writer (void[] dst)
                {
                      // cap bytes written
                        uint len = dst.length & mask;
                        if (len > count)
                            len = count;

                        dst [0..len] = src [0..len];
                        mutate (dst, len);
                        return len;
                }

                while (count)
                      {
                      //flush if we used all buffer space
                      if (count -= buffer.write (&writer))
                          if (buffer.getConduit)
                              buffer.flush ();
                          else
                             if (! buffer.grow (count))
                                   throw ovf;
                      }                          
        }

}

