/*******************************************************************************

        @file Writer.d
        

        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, March 2004      
        @author         Kris

*******************************************************************************/

module mango.io.Writer;

public import mango.io.AbstractWriter;

/*******************************************************************************

        Binary data Writer. Writers provide the means to append formatted 
        data to an IBuffer, and expose a convenient method of handling a
        variety of data types. In addition to writing native types such
        as integer and char[], writers also process any class which has
        implemented the IWritable interface (one method).

        See AbstractWriter for usage examples.

*******************************************************************************/

class Writer : AbstractWriter
{  
        /***********************************************************************
        
                Construct a Writer upon the provided IBuffer. All formatted
                output will be directed to this buffer.

        ***********************************************************************/

        this (IBuffer buffer)
        {
                super (buffer);

                encode.int1    = 
                encode.int8    = 
                encode.int8u   =
                encode.int16   = 
                encode.int16u  = 
                encode.int32   = 
                encode.int32u  =
                encode.int64   =
                encode.int64u  =
                encode.float32 =
                encode.float64 =
                encode.float80 =
                encode.char8    = 
                encode.char16   = 
                encode.char32   = &write;
        }
     
        /***********************************************************************
        
                Construct a Writer on the buffer associated with the given
                conduit.

        ***********************************************************************/

        this (IConduit conduit)
        {
                this (conduit.createBuffer);
        }

        /***********************************************************************
        
        ***********************************************************************/

        final void write (void* src, uint count)
        {
                buffer.append (src[0..count]);
        }
}
