/*******************************************************************************

        @file DateLayout.d

        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      
        @version        Initial version, May 2004
        @author         Kris


*******************************************************************************/

module mango.log.DateLayout;

private import  std.c.time,
                std.c.stdio;

private import  mango.log.Event,
                mango.log.Layout;

/*******************************************************************************

        A layout with ISO-8601 date information prefixed to each message.
       
*******************************************************************************/

public class DateLayout : Layout
{
        version (Win32)
                 alias _snprintf snprintf;

        /***********************************************************************
                
                Format message attributes into an output buffer and return
                the populated portion.

        ***********************************************************************/

        char[] header (Event event)
        {
                tm t;
                time_t seconds = event.getEpochSeconds();

                // we synchronize due to non-reentrant call
                synchronized (this)
                              t = *gmtime (&seconds);
                
                char[] scratch = event.scratch.content;

                int i = snprintf (scratch, scratch.length, "%04d-%02d-%02d %02d:%02d:%02d,%03ld %-6.*s %.*s - ",
                                  t.tm_year + 1900, 
                                  t.tm_mon + 1, 
                                  t.tm_mday, 
                                  t.tm_hour, 
                                  t.tm_min, 
                                  t.tm_sec, 
                                  event.getMilliSeconds(),
                                  levelString (event),
                                  event.getName
                                  );            
                return scratch[0..i];
        }
}


